subroutine compsums1w(t,x,rng,nk,or,Wmc,Zmc,theta0,m,C,D,se,nu_re,J_re, &
					I,n,Nx,Nmc,Nmc2,p,q,r,s,d1,d2, &	! the rest are output arguments
					Sww,Szz,SBB,SBr,Srx2,SBBu2,SuBr,SBBv2,SvBr,fi, &
					What,Zhat,What2,Zhat2,errflag)
					
! Computes sums of expectations for Matlab function MLHR1W (via gateway function compsums1w_gtw)
! external subroutines used: jupp2, bspl2, hwarpinv2

! NOTE: MC sample vectors W = [eta;u] and Z = [xi;v], so d1=r+p and d2=r+q

!declare arguments
implicit none
double precision, intent(in) :: t(Nx), x(Nx), rng(2), Wmc(d1,Nmc), Zmc(d2,Nmc), &
	theta0(1,r), m(s), C(s,p), D(s,q), se, nu_re(n), J_re(I)
integer, intent(in) :: nk, or, I, n, Nx, Nmc, Nmc2, p, q, r, s, d1, d2
double precision, intent(out) :: Sww(d1,d1), Szz(d2,d2), SBB(s,s), SBr(s), &
	Srx2, SBBu2(s,s,p), SuBr(s,p), SBBv2(s,s,q), SvBr(s,q), fi(I), &
	What(I,d1), Zhat(n,d2), What2(d1,d1,I), Zhat2(d2,d2,n)
integer, intent(out) :: errflag
!declare locals
double precision :: knots(nk+2), t_star(Nx), B(Nx,s), mux(Nx)
double precision :: tau0(1,r), vtau0(r), vtheta0(r), taumc(Nmc2,r), vtaumc(r), thetamc(Nmc2,r)
double precision :: BtB(s,s), Btr(s)
double precision :: Nd, se2, fcond, g(n), h, gprod
double precision :: Izz(d2,d2), Iz(1,d2), IBB(s,s), IBr(s), Irx2, IBBu2(s,s,p), &
	IuBr(s,p), IBBv2(s,s,q), IvBr(s,q)
integer :: nucum, i0, j0, k, j1, j2, ii, jj, nu(n), J(I), Jcum
double precision, parameter :: pi = 3.141592654d0


! Begin
errflag = 0
nu = int(nu_re)
J = int(J_re)
forall (i0=1:nk+2) knots(i0) = rng(1) + (rng(2)-rng(1))*dble(i0-1)/dble(nk+1)
Nd = dble(Nmc)
se2 = se**2
 call jupp2(theta0,rng(1),rng(2),"inverse",tau0,errflag,1,r)
if (errflag==1) return
vtheta0 = theta0(1,:)
vtau0 = tau0(1,:)

! MC theta's
do j1 = 1,Nmc	! w loop
	do j2 = 1,Nmc	! z loop
		thetamc((j1-1)*Nmc+j2,:) = vtheta0 + Wmc(1:r,j1) + Zmc(1:r,j2)
	end do
end do
 call jupp2(thetamc,rng(1),rng(2),"inverse",taumc,errflag,Nmc2,r)


! Likelihood
fi = 0.d0
do j1 = 1,Nmc	! w loop
	g = 0.d0
	k = 0
	nucum = 0
	Jcum = 0
	do i0 = 1,I
		do j0 = 1,J(i0)
			k = k+1				
			do j2 = 1,Nmc	! z loop
				vtaumc = taumc((j1-1)*Nmc+j2,:)
				call hwarpinv2(t(nucum+1:nucum+nu(k)),vtaumc,vtau0,rng,t_star(nucum+1:nucum+nu(k)),errflag,nu(k),r)
				if (errflag==1) return
				call bspl2(t_star(nucum+1:nucum+nu(k)),or,knots,0,B(nucum+1:nucum+nu(k),:),nu(k),nk+2)
				mux(nucum+1:nucum+nu(k)) = matmul(B(nucum+1:nucum+nu(k),:),m+matmul(C,Wmc(r+1:r+p,j1))+matmul(D,Zmc(r+1:r+q,j2)))
				fcond = dexp(-0.5d0*sum((x(nucum+1:nucum+nu(k))-mux(nucum+1:nucum+nu(k)))**2)/se2)/dsqrt(2.d0*pi*se2)**nu(k)
				g(k) = g(k) + fcond/Nd
			end do
			nucum = nucum + nu(k)
		end do
		fi(i0) = fi(i0) + product(g(Jcum+1:Jcum+J(i0)))/Nd
		Jcum = Jcum + J(i0)
	end do
end do


! Sums of expectations
Szz = 0.d0
SBB = 0.d0
SBr = 0.d0
Srx2 = 0.d0
SBBu2 = 0.d0
SuBr = 0.d0
SBBv2 = 0.d0
SvBr = 0.d0
Sww = 0.d0
What = 0.d0
Zhat = 0.d0
What2 = 0.d0
Zhat2 = 0.d0
do j1 = 1,Nmc	! w loop
	g = 0.d0
	do i0 = 1,I
		if (i0==1) then
			Jcum = 0
		else
			Jcum = sum(J(1:i0-1))
		end if
		do j0 = 1,J(i0)
			k = Jcum + j0
			if (k==1) then
				nucum = 0
			else
				nucum = sum(nu(1:k-1))
			end if
			do j2 = 1,Nmc	! z loop
				vtaumc = taumc((j1-1)*Nmc+j2,:)
				call hwarpinv2(t(nucum+1:nucum+nu(k)),vtaumc,vtau0,rng,t_star(nucum+1:nucum+nu(k)),errflag,nu(k),r)
				if (errflag==1) return
				call bspl2(t_star(nucum+1:nucum+nu(k)),or,knots,0,B(nucum+1:nucum+nu(k),:),nu(k),nk+2)
				mux(nucum+1:nucum+nu(k)) = matmul(B(nucum+1:nucum+nu(k),:),m+matmul(C,Wmc(r+1:r+p,j1))+matmul(D,Zmc(r+1:r+q,j2)))
				fcond = dexp(-0.5d0*sum((x(nucum+1:nucum+nu(k))-mux(nucum+1:nucum+nu(k)))**2)/se2)/dsqrt(2.d0*pi*se2)**nu(k)
				g(k) = g(k) + fcond/Nd
			end do
		end do
		gprod = product(g(Jcum+1:Jcum+J(i0)))
		do j0 = 1,J(i0)
			k = Jcum + j0
			if (k==1) then
				nucum = 0
			else
				nucum = sum(nu(1:k-1))
			end if
			Izz = 0.d0
			Iz = 0.d0
			IBB = 0.d0
			IBr = 0.d0
			Irx2 = 0.d0
			IBBu2 = 0.d0
			IuBr = 0.d0
			IBBv2 = 0.d0
			IvBr = 0.d0
			do j2 = 1,Nmc	! z loop
				vtaumc = taumc((j1-1)*Nmc+j2,:)
				call hwarpinv2(t(nucum+1:nucum+nu(k)),vtaumc,vtau0,rng,t_star(nucum+1:nucum+nu(k)),errflag,nu(k),r)
				if (errflag==1) return
				call bspl2(t_star(nucum+1:nucum+nu(k)),or,knots,0,B(nucum+1:nucum+nu(k),:),nu(k),nk+2)
				mux(nucum+1:nucum+nu(k)) = matmul(B(nucum+1:nucum+nu(k),:),m+matmul(C,Wmc(r+1:r+p,j1))+matmul(D,Zmc(r+1:r+q,j2)))
				fcond = dexp(-0.5d0*sum((x(nucum+1:nucum+nu(k))-mux(nucum+1:nucum+nu(k)))**2)/se2)/dsqrt(2.d0*pi*se2)**nu(k)
				forall(ii=1:d2,jj=1:d2) Izz(ii,jj) = Izz(ii,jj) + Zmc(ii,j2)*Zmc(jj,j2)*(fcond/Nd)
				Iz(1,:) = Iz(1,:) + Zmc(:,j2)*(fcond/Nd)
    	    	BtB = matmul(transpose(B(nucum+1:nucum+nu(k),:)),B(nucum+1:nucum+nu(k),:))
    	    	IBB = IBB + BtB*(fcond/Nd)
    	    	Btr = matmul(transpose(B(nucum+1:nucum+nu(k),:)), &
    	    		x(nucum+1:nucum+nu(k))-matmul(B(nucum+1:nucum+nu(k),:),matmul(C,Wmc(r+1:r+p,j1))+matmul(D,Zmc(r+1:r+q,j2))))
				IBr = IBr + Btr*(fcond/Nd)
				Irx2 = Irx2 + sum((x(nucum+1:nucum+nu(k))-mux(nucum+1:nucum+nu(k)))**2)*(fcond/Nd)
    	    	Btr = matmul(transpose(B(nucum+1:nucum+nu(k),:)),x(nucum+1:nucum+nu(k))-mux(nucum+1:nucum+nu(k)))
				forall(jj = 1:p) IBBu2(:,:,jj) = IBBu2(:,:,jj) + BtB*(Wmc(r+jj,j1)**2)*(fcond/Nd)
				forall(jj = 1:p) IuBr(:,jj) = IuBr(:,jj) + Btr*Wmc(r+jj,j1)*(fcond/Nd)
				forall(jj = 1:q) IBBv2(:,:,jj) = IBBv2(:,:,jj) + BtB*(Zmc(r+jj,j2)**2)*(fcond/Nd)
				forall(jj = 1:q) IvBr(:,jj) = IvBr(:,jj) + Btr*Zmc(r+jj,j2)*(fcond/Nd)
			end do
			if (g(k)>0.d0) then
				forall(ii=1:d2,jj=1:d2) Szz(ii,jj) = Szz(ii,jj) + Izz(ii,jj)*(gprod/g(k))/(fi(i0)*Nd)
				Zhat(k,:) = Zhat(k,:) + Iz(1,:)*(gprod/g(k))/(fi(i0)*Nd)
				forall(ii=1:d2,jj=1:d2) Zhat2(ii,jj,k) = Zhat2(ii,jj,k) + Izz(ii,jj)*(gprod/g(k))/(fi(i0)*Nd)
    	    	SBB = SBB + IBB*(gprod/g(k))/(fi(i0)*Nd)
				SBr = SBr + IBr*(gprod/g(k))/(fi(i0)*Nd)
				Srx2 = Srx2 + Irx2*(gprod/g(k))/(fi(i0)*Nd)
				forall(jj = 1:p) SBBu2(:,:,jj) = SBBu2(:,:,jj) + IBBu2(:,:,jj)*(gprod/g(k))/(fi(i0)*Nd)
				forall(jj = 1:p) SuBr(:,jj) = SuBr(:,jj) + IuBr(:,jj)*(gprod/g(k))/(fi(i0)*Nd)
				forall(jj = 1:q) SBBv2(:,:,jj) = SBBv2(:,:,jj) + IBBv2(:,:,jj)*(gprod/g(k))/(fi(i0)*Nd)
				forall(jj = 1:q) SvBr(:,jj) = SvBr(:,jj) + IvBr(:,jj)*(gprod/g(k))/(fi(i0)*Nd)
			end if
		end do
		if (fi(i0)>0.d0) then
			forall(ii=1:d1,jj=1:d1) Sww(ii,jj) = Sww(ii,jj) + Wmc(ii,j1)*Wmc(jj,j1)*(gprod/Nd)/fi(i0)
			What(i0,:) = What(i0,:) + Wmc(:,j1)*(gprod/Nd)/fi(i0)
			forall(ii=1:d1,jj=1:d1) What2(ii,jj,i0) = What2(ii,jj,i0) + Wmc(ii,j1)*Wmc(jj,j1)*(gprod/Nd)/fi(i0)
		end if
	end do
end do


end subroutine compsums1w

