C     Gateway subroutine
C
C     compsums1w0_gtw.F
C
C     Invokes Fortran COMPSUMS1W0 routine for MLHR1W0 (initial one-way models)
C      
C	  January 2013
C
C   -------------------------------------------------------------
C
      subroutine mexFunction(nlhs, plhs, nrhs, prhs)

      integer plhs(*), prhs(*)
      integer mxGetPr, mxCreateDoubleMatrix, mxCreateNumericArray
      integer mxClassIDFromClassName
      integer nlhs, nrhs
      integer mxGetM, mxGetN
      integer t,x,rng,nkv,orv,Wmc,Zmc,theta0,m,se,nu,J
      integer Sww,Szz,SBB,SBr,Srx2,fi,W,Z,W2,Z2
      integer nk,or,Nx,Nmc,Nmc2,r,s,n,I
      integer n_t,n_x,n_rng,n_nkv,n_orv,n_Wmc,n_Zmc,n_theta0,
     .        n_m,n_se,n_nu,n_J
      integer m_t,m_x,m_rng,m_nkv,m_orv,m_Wmc,m_Zmc,m_theta0,
     .        m_m,m_se,m_nu,m_J
      integer errflag, dims(3)

C     Check number of inputs and outputs
      if(nrhs .ne. 12) then
         call mexErrMsgTxt('12 inputs required.')
      elseif(nlhs .ne. 10) then
         call mexErrMsgTxt('10 outputs required.')
      endif

C     Load input pointers
	  t = mxGetPr(prhs(1))
	  x = mxGetPr(prhs(2))
	  rng = mxGetPr(prhs(3))
	  nkv = mxGetPr(prhs(4))
	  orv = mxGetPr(prhs(5))
	  Wmc = mxGetPr(prhs(6))
	  Zmc = mxGetPr(prhs(7))
      theta0 = mxGetPr(prhs(8))
	  m = mxGetPr(prhs(9))
	  se = mxGetPr(prhs(10))
      nu = mxGetPr(prhs(11))
	  J = mxGetPr(prhs(12))

C     Check input dimensions        
      m_t = mxGetM(prhs(1))
      n_t = mxGetN(prhs(1))
      if((m_t.gt.1).and.(n_t.gt.1)) then
         call mexErrMsgTxt('Input T must be a vector.')
      endif
      Nx = max(m_t,n_t)
      m_x = mxGetM(prhs(2))
      n_x = mxGetN(prhs(2))
      if((m_x.gt.1).and.(n_x.gt.1)) then
         call mexErrMsgTxt('Input X must be a vector.')
      endif
      if(max(m_x,n_x).ne.Nx) then
         call mexErrMsgTxt('Length(X) must equal length(T).')
      endif
      m_rng = mxGetM(prhs(3))
      n_rng = mxGetN(prhs(3))
      if((m_rng.gt.1).and.(n_rng.gt.1)) then
         call mexErrMsgTxt('Input RNG must be a vector.')
      endif
      if(max(m_rng,n_rng).gt.2) then
         call mexErrMsgTxt('Length(RNG) must be 2.')
      endif
      m_nkv = mxGetM(prhs(4))
      n_nkv = mxGetN(prhs(4))
      if((m_nkv.gt.1).and.(n_nkv.gt.1)) then
         call mexErrMsgTxt('Input NKV must be a vector.')
      endif
      nk = max(m_nkv,n_nkv)
      m_orv = mxGetM(prhs(5))
      n_orv = mxGetN(prhs(5))
      if((m_orv.gt.1).and.(n_orv.gt.1)) then
         call mexErrMsgTxt('Input ORV must be a vector.')
      endif
      or = max(m_orv,n_orv)
      Nmc = mxGetN(prhs(6))
      Nmc2 = Nmc**2
      m_theta0 = mxGetM(prhs(8))
      n_theta0 = mxGetN(prhs(8))
      if((m_theta0.gt.1).and.(n_theta0.gt.1)) then
         call mexErrMsgTxt('Input THETA0 must be a vector.')
      endif
      r = max(m_theta0,n_theta0)
      m_m = mxGetM(prhs(9))
      n_m = mxGetN(prhs(9))
      if((m_m.gt.1).and.(n_m.gt.1)) then
         call mexErrMsgTxt('Input M must be a vector.')
      endif
      s = max(m_m,n_m)
      if(s.ne.(nk+or)) then
         call mexErrMsgTxt('Inputs M, NK and OR inconsistent.')
      endif
      m_se = mxGetM(prhs(10))
      n_se = mxGetN(prhs(10))
      if((m_se.gt.1).or.(n_se.gt.1)) then
         call mexErrMsgTxt('Input SE must be a scalar.')
      endif
      m_nu = mxGetM(prhs(11))
      n_nu = mxGetN(prhs(11))
      if((m_nu.gt.1).and.(n_nu.gt.1)) then
         call mexErrMsgTxt('Input NU must be a vector.')
      endif
      n = max(m_nu,n_nu)
      m_J = mxGetM(prhs(12))
      n_J = mxGetN(prhs(12))
      if((m_J.gt.1).and.(n_J.gt.1)) then
         call mexErrMsgTxt('Input J must be a vector.')
      endif
      I = max(m_J,n_J)

C     Create arrays for outputs
      plhs(1) = mxCreateDoubleMatrix(r,r,0)
      Sww = mxGetPr(plhs(1))
      plhs(2) = mxCreateDoubleMatrix(r,r,0)
      Szz = mxGetPr(plhs(2))
      plhs(3) = mxCreateDoubleMatrix(s,s,0)
      SBB = mxGetPr(plhs(3))
      plhs(4) = mxCreateDoubleMatrix(s,1,0)
      SBr = mxGetPr(plhs(4))
      plhs(5) = mxCreateDoubleMatrix(1,1,0)
      Srx2 = mxGetPr(plhs(5))
      plhs(6) = mxCreateDoubleMatrix(I,1,0)
      fi = mxGetPr(plhs(6))
      plhs(7) = mxCreateDoubleMatrix(I,r,0)
      W = mxGetPr(plhs(7))
      plhs(8) = mxCreateDoubleMatrix(n,r,0)
      Z = mxGetPr(plhs(8))
      dims(1) = r
      dims(2) = r
      dims(3) = I
      plhs(9) = mxCreateNumericArray(3,dims,
     .                      mxClassIDFromClassName("double"),0)
      W2 = mxGetPr(plhs(9))
      dims(1) = r
      dims(2) = r
      dims(3) = n
      plhs(10) = mxCreateNumericArray(3,dims,
     .                      mxClassIDFromClassName("double"),0)
      Z2 = mxGetPr(plhs(10))

C     Call the computational subroutine.
      call compsums1w0(%val(t),%val(x),%val(rng),nk,or,%val(Wmc),
     . %val(Zmc),%val(theta0),%val(m),%val(se),
     . %val(nu),%val(J),I,n,Nx,Nmc,Nmc2,r,s,
     . %val(Sww),%val(Szz),%val(SBB),%val(SBr),%val(Srx2),
     . %val(fi),%val(W),%val(Z),%val(W2),%val(Z2),errflag)

	  if (errflag.eq.1) then
	  	call mexErrMsgTxt('COMPSUMS1W0 is returning an error flag.')
	  end if
	  
      return
      end

